package sun.util.calendar;

import java.util.TimeZone;
import java.security.PrivilegedActionException;
import java.io.IOException;
import java.io.FileInputStream;
import java.security.PrivilegedExceptionAction;
import java.io.File;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

public abstract class CalendarSystem
{
    private static volatile boolean initialized;
    private static ConcurrentMap<String, String> names;
    private static ConcurrentMap<String, CalendarSystem> calendars;
    private static final String PACKAGE_NAME = "sun.util.calendar.";
    private static final String[] namePairs;
    private static final Gregorian GREGORIAN_INSTANCE;
    
    public CalendarSystem() {
    }
    
    private static void initNames() {
        ConcurrentHashMap<Object, String> names = new ConcurrentHashMap();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < CalendarSystem.namePairs.length; i += 2) {
            sb.setLength(0);
            names.put(CalendarSystem.namePairs[i], sb.append("sun.util.calendar.").append(CalendarSystem.namePairs[i + 1]).toString());
        }
        synchronized (CalendarSystem.class) {
            if (!CalendarSystem.initialized) {
                CalendarSystem.names = names;
                CalendarSystem.calendars = new ConcurrentHashMap();
                CalendarSystem.initialized = true;
            }
        }
    }
    
    public static Gregorian getGregorianCalendar() {
        return CalendarSystem.GREGORIAN_INSTANCE;
    }
    
    public static CalendarSystem forName(String anObject) {
        if ("gregorian".equals(anObject)) {
            return CalendarSystem.GREGORIAN_INSTANCE;
        }
        if (!CalendarSystem.initialized) {
            initNames();
        }
        CalendarSystem calendarSystem = (CalendarSystem)CalendarSystem.calendars.get(anObject);
        if (calendarSystem != null) {
            return calendarSystem;
        }
        String className = (String)CalendarSystem.names.get(anObject);
        if (className == null) {
            return null;
        }
        LocalGregorianCalendar localGregorianCalendar;
        if (className.endsWith("LocalGregorianCalendar")) {
            localGregorianCalendar = LocalGregorianCalendar.getLocalGregorianCalendar(anObject);
        }
        else {
            try {
                localGregorianCalendar = (CalendarSystem)Class.forName(className).newInstance();
            }
            catch (Exception cause) {
                throw new InternalError(cause);
            }
        }
        if (localGregorianCalendar == null) {
            return null;
        }
        CalendarSystem calendarSystem2 = (CalendarSystem)CalendarSystem.calendars.putIfAbsent(anObject, localGregorianCalendar);
        return (calendarSystem2 == null) ? localGregorianCalendar : calendarSystem2;
    }
    
    public static Properties getCalendarProperties() throws IOException {
        Properties properties;
        try {
            properties = (Properties)AccessController.doPrivileged(new PrivilegedExceptionAction<Properties>() {
                
                @Override
                public Properties run() throws IOException {
                    Properties properties = new Properties();
                    try (final FileInputStream inStream = new FileInputStream(fname)) {
                        properties.load(inStream);
                    }
                    return properties;
                }
            });
        }
        catch (PrivilegedActionException ex) {
            Throwable cause = ex.getCause();
            if ((cause instanceof IOException)) {
                throw (IOException)cause;
            }
            if ((cause instanceof IllegalArgumentException)) {
                throw (IllegalArgumentException)cause;
            }
            throw new InternalError(cause);
        }
        return properties;
    }
    
    public abstract String getName();
    
    public abstract CalendarDate getCalendarDate();
    
    public abstract CalendarDate getCalendarDate(long p0);
    
    public abstract CalendarDate getCalendarDate(long p0, CalendarDate p1);
    
    public abstract CalendarDate getCalendarDate(long p0, TimeZone p1);
    
    public abstract CalendarDate newCalendarDate();
    
    public abstract CalendarDate newCalendarDate(TimeZone p0);
    
    public abstract long getTime(CalendarDate p0);
    
    public abstract int getYearLength(CalendarDate p0);
    
    public abstract int getYearLengthInMonths(CalendarDate p0);
    
    public abstract int getMonthLength(CalendarDate p0);
    
    public abstract int getWeekLength();
    
    public abstract Era getEra(String p0);
    
    public abstract Era[] getEras();
    
    public abstract void setEra(CalendarDate p0, String p1);
    
    public abstract CalendarDate getNthDayOfWeek(int p0, int p1, CalendarDate p2);
    
    public abstract CalendarDate setTimeOfDay(CalendarDate p0, int p1);
    
    public abstract boolean validate(CalendarDate p0);
    
    public abstract boolean normalize(CalendarDate p0);
    
    static {
        CalendarSystem.initialized = false;
        namePairs = new String[] { "gregorian", "Gregorian", "japanese", "LocalGregorianCalendar", "julian", "JulianCalendar" };
        GREGORIAN_INSTANCE = new Gregorian();
    }
}
